function md=setmask2(md,landname,floatingicename,groundedicename)
%GEOGRAPHY2 - establish land, ice sheet and ice shelf areas in a domains.
%
%   Usage:
%      md=setmask2(md,landname,floatingicename,groundedicename)
%
%   Examples:
%      md=setmask2(md,'LandName.exp','Iceshelves.exp','Islands.exp');

%Get assigned fields
x=md.x;
y=md.y;
elements=md.elements;

%recover elements and nodes on land.
if ischar(landname),
	[vertexonland,elementonland]=ContourToMesh(elements,x,y,landname,'element and node',2);
elseif isfloat(landname),
	if size(landname,1)~=md.numberofelements,
		error('Landname for area must be of same size as number of elements in model');
	end
	elementonland=landname;
	vertexonland=zeros(md.numberofnodes,1);
	vertexonland(md.elements(find(elementonland),:))=1;
else
	error('Invalid area option option');
end

%Now, build the connectivity tables for this mesh.
if size(md.nodeconnectivity,1)~=md.numberofnodes,
	md.nodeconnectivity=NodeConnectivity(md.elements,md.numberofnodes);
end
if size(md.elementconnectivity,1)~=md.numberofelements,
	md.elementconnectivity=ElementConnectivity(md.elements,md.nodeconnectivity);
end

%any element with 3 nodes on land should be on land:
elementsonwater=find(~elementonland);
wrongelements=elementsonwater(find(( vertexonland(md.elements(elementsonwater,1)) + vertexonland(md.elements(elementsonwater,2)) + vertexonland(md.elements(elementsonwater,3)) ...
                  )==3));
elementonland(wrongelements)=1;

%any element with its barycentre on land should be on land: (only if landname is an expfile)
if ischar(landname),
weights={[1;1;1],[2;1;1],[1;2;1],[1;1;2]};
	for i=1:length(weights),
		xelem=x(md.elements)*weights{i}/sum(weights{i});
		yelem=y(md.elements)*weights{i}/sum(weights{i});
	end
	baryonland=ContourToNodes(xelem,yelem,landname,1);
	pos=find(~baryonland); elementonland(pos)=0;
	pos=find(baryonland); elementonland(pos)=1;
end

%figure out which elements on land are actually in the middle of the ocean!
pos1=find(elementonland); 
connectedtoland=md.elementconnectivity(pos1,:);
pos=find(connectedtoland); connectedtoland(pos)=1-elementonland(connectedtoland(pos));
connectedtolandsum=sum(connectedtoland,2);
waterelements=pos1(find(connectedtolandsum==3));
elementonland(waterelements)=0;

%figure out which elements on water  are actually in the middle of the land!
pos1=find(~elementonland); 
connectedtowater=md.elementconnectivity(pos1,:);
pos=find(connectedtowater); connectedtowater(pos)=elementonland(connectedtowater(pos));
connectedtowatersum=sum(connectedtowater,2);
landelements=pos1(find(connectedtowatersum==3));
elementonland(landelements)=1;

%recover arrays of ice shelf nodes and elements, and ice sheet nodes and elements.
elementonfloatingice=FlagElements(md,floatingicename);
elementongroundedice=FlagElements(md,groundedicename);

%Because groundedice nodes and elements can be included into an floatingice, we need to update. Remember, all the previous 
%arrays come from domain outlines that can intersect one another: 
vertexonfloatingice=zeros(md.numberofnodes,1);
vertexongroundedice=zeros(md.numberofnodes,1);
elementonfloatingice=double((elementonfloatingice & ~elementongroundedice));
elementongroundedice=double(~elementonfloatingice);
vertexonfloatingice(md.elements(find(elementonfloatingice),:))=1;
vertexongroundedice(md.elements(find(elementongroundedice),:))=1;

%now correct, so that none of the floatingice and groundedice elements and nodes are in the water.
pos=find(~elementonland);
elementonfloatingice(pos)=0; 
elementongroundedice(pos)=0;

pos=find(~vertexonland);
vertexonfloatingice(pos)=0; 
vertexongroundedice(pos)=0;

%create vertexonwater and elementonwater: 
vertexonwater=double(~vertexonland);
elementonwater=double(~elementonland);

%correct for islands:
vertexonfloatingice=double(vertexonfloatingice & ~vertexongroundedice);
elementonfloatingice=double(elementonfloatingice & ~elementongroundedice);

%now, groundedices are everything except iceshelves and water
vertexongroundedice=double(~vertexonfloatingice & ~vertexonwater);
elementongroundedice=double(~elementonfloatingice & ~elementonwater);

%Deal with segments on neumann:

%Get current connectivity
elementconnectivity=md.elementconnectivity;

%put 0 for elements on water
pos=find(elementconnectivity);
elementconnectivity(pos)=elementconnectivity(pos).*(~elementonwater(elementconnectivity(pos)));

%put line of ones for elements on water
pos=find(elementonwater);
elementconnectivity(pos,:)=1;% line of ones for elements on water so they won't be considered

%resort lines (zeros must be at the last column for findsegments)
elementconnectivity=sort(elementconnectivity,2,'descend');

%call findsegments to build segment using THIS conectivity
md.segments=findsegments(md,'elementconnectivity',elementconnectivity);

%some final checks: 
%check that no node thinks it's on an ice shelf or ice sheet, and lies actually in the middle of the water.
nodesgrounded=find(~vertexonwater);
lengthconnectivity=size(md.nodeconnectivity,2);
groundedcounters=md.nodeconnectivity(nodesgrounded,lengthconnectivity);
groundedconnectivity=md.nodeconnectivity(nodesgrounded,1:lengthconnectivity-1);
pos=find(groundedconnectivity);
groundedconnectivity(pos)=elementonwater(groundedconnectivity(pos));
groundedsum=sum(groundedconnectivity,2);
errorflags=find(groundedsum==groundedcounters);
errornodes=nodesgrounded(errorflags);

vertexonwater(errornodes)=1;
vertexongroundedice(errornodes)=0;
vertexonfloatingice(errornodes)=0;

%Return: 
md.mask.vertexonfloatingice=vertexonfloatingice;
md.mask.elementonfloatingice=elementonfloatingice;
md.mask.vertexonwater=vertexonwater;
md.mask.elementonwater=elementonwater;
md.mask.vertexongroundedice=vertexongroundedice;
md.mask.elementongroundedice=elementongroundedice;

md.segmentmarkers(:)=1;
